==> hpc_beeflow-0.1.4.tar.gz: hpc_beeflow-0.1.4/beeflow/wf_manager/resources/wf_actions.py <==
"""This module contains the workflow action endpoints."""

from flask import make_response, jsonify
from flask_restful import Resource, reqparse
from beeflow.common import log as bee_logging
from beeflow.wf_manager.resources import wf_utils

from beeflow.common.db import wfm_db
from beeflow.common.db.bdb import connect_db

log = bee_logging.setup(__name__)
db_path = wf_utils.get_db_path()


class WFActions(Resource):
    """Class to perform actions on existing workflows."""

    def __init__(self):
        """Initialize with passed json object."""
        self.reqparse = reqparse.RequestParser()

    def post(self, wf_id):
        """Start workflow. Send ready tasks to the task manager."""
        db = connect_db(wfm_db, db_path)
        wfi = wf_utils.get_workflow_interface(wf_id)
        state = wfi.get_workflow_state()
        if state in ('RUNNING', 'PAUSED', 'COMPLETED'):
            resp = make_response(jsonify(msg='Cannot start workflow it is '
                                 f'{state.lower()}.',
                                         status='ok'), 200)
            return resp
        wfi.execute_workflow()
        tasks = wfi.get_ready_tasks()
        wf_utils.schedule_submit_tasks(wf_id, tasks)
        wf_id = wfi.workflow_id
        wf_utils.update_wf_status(wf_id, 'Running')
        db.workflows.update_workflow_state(wf_id, 'Running')
        resp = make_response(jsonify(msg='Started workflow!', status='ok'), 200)
        return resp

    @staticmethod
    def get(wf_id):
        """Check the database for the current status of all tasks."""
        db = connect_db(wfm_db, db_path)
        tasks = db.workflows.get_tasks(wf_id)
        tasks_status = []
        if not tasks:
            log.info(f"Bad query for wf {wf_id}.")
            wf_status = 'No workflow with that ID is currently loaded'
            tasks_status.append('Unavailable')
            resp = make_response(jsonify(tasks_status=tasks_status,
                                 wf_status=wf_status, status='not found'), 404)

        for task in tasks:
            tasks_status.append(f"{task.name}--{task.state}")
        tasks_status = '\n'.join(tasks_status)
        wf_status = wf_utils.read_wf_status(wf_id)

        resp = make_response(jsonify(tasks_status=tasks_status,
                             wf_status=wf_status, status='ok'), 200)
        return resp

    @staticmethod
    def delete(wf_id):
        """Cancel the workflow. Lets current tasks finish running."""
        db = connect_db(wfm_db, db_path)
        wfi = wf_utils.get_workflow_interface(wf_id)
        # Remove all tasks currently in the database
        if wfi.workflow_loaded():
            wfi.finalize_workflow()
        wf_utils.update_wf_status(wf_id, 'Cancelled')
        db.workflows.update_workflow_state(wf_id, 'Cancelled')
        db.workflows.delete_workflow(wf_id)
        log.info("Workflow cancelled")
        resp = make_response(jsonify(status='Cancelled'), 202)
        return resp

    def patch(self, wf_id):
        """Pause or resume workflow."""
        db = connect_db(wfm_db, db_path)
        self.reqparse.add_argument('option', type=str, location='json')
        option = self.reqparse.parse_args()['option']

        wfi = wf_utils.get_workflow_interface(wf_id)
        wf_state = wfi.get_workflow_state()
        if option == 'pause' and wf_state == 'RUNNING':
            wfi.pause_workflow()
            wf_utils.update_wf_status(wf_id, 'Paused')
            db.workflows.update_workflow_state(wf_id, 'Paused')
            log.info("Workflow Paused")
            resp = make_response(jsonify(status='Workflow Paused'), 200)
        elif option == 'resume' and wf_state == 'PAUSED':
            wfi.resume_workflow()
            tasks = wfi.get_ready_tasks()
            wf_utils.schedule_submit_tasks(wf_id, tasks)
            wf_utils.update_wf_status(wf_id, 'Running')
            db.workflows.update_workflow_state(wf_id, 'Running')
            log.info("Workflow Resumed")
            resp = make_response(jsonify(status='Workflow Resumed'), 200)
        else:
            resp_msg = f'Cannot {option} workflow. It is currently {wf_state.lower()}.'
            log.info(resp_msg)
            resp = make_response(jsonify(status=resp_msg), 200)
        return resp

==> hpc_beeflow-0.1.6.tar.gz: hpc_beeflow-0.1.6/beeflow/wf_manager/resources/wf_actions.py <==
"""This module contains the workflow action endpoints."""

from flask import make_response, jsonify
from flask_restful import Resource, reqparse
from beeflow.common import log as bee_logging
from beeflow.wf_manager.resources import wf_utils

from beeflow.common.db import wfm_db
from beeflow.common.db.bdb import connect_db
from beeflow.wf_manager.common import dep_manager

log = bee_logging.setup(__name__)
db_path = wf_utils.get_db_path()


class WFActions(Resource):
    """Class to perform actions on existing workflows."""

    def __init__(self):
        """Initialize with passed json object."""
        self.reqparse = reqparse.RequestParser()

    def post(self, wf_id):
        """Start workflow. Send ready tasks to the task manager."""
        db = connect_db(wfm_db, db_path)
        if wf_utils.start_workflow(wf_id):
            db.workflows.update_workflow_state(wf_id, 'Running')
            resp = make_response(jsonify(msg='Started workflow!', status='ok'), 200)
        else:
            resp_body = jsonify(msg='Cannot start workflow it is {state.lower()}.', status='ok')
            resp = make_response(resp_body, 200)
        return resp

    @staticmethod
    def get(wf_id):
        """Check the database for the current status of all tasks."""
        db = connect_db(wfm_db, db_path)
        tasks = db.workflows.get_tasks(wf_id)
        tasks_status = []
        if not tasks:
            log.info(f"Bad query for wf {wf_id}.")
            wf_status = 'No workflow with that ID is currently loaded'
            tasks_status.append('Unavailable')
            resp = make_response(jsonify(tasks_status=tasks_status,
                                 wf_status=wf_status, status='not found'), 404)

        for task in tasks:
            tasks_status.append(f"{task.name}--{task.state}")
        tasks_status = '\n'.join(tasks_status)
        wf_status = db.workflows.get_workflow_state(wf_id)

        resp = make_response(jsonify(tasks_status=tasks_status,
                             wf_status=wf_status, status='ok'), 200)
        return resp

    @staticmethod
    def delete(wf_id):
        """Cancel the workflow. Lets current tasks finish running."""
        db = connect_db(wfm_db, db_path)
        wfi = wf_utils.get_workflow_interface(wf_id)
        # Remove all tasks currently in the database
        if wfi.workflow_loaded():
            wfi.finalize_workflow()
        wf_utils.update_wf_status(wf_id, 'Cancelled')
        db.workflows.update_workflow_state(wf_id, 'Cancelled')
        log.info("Workflow cancelled")
        log.info("Shutting down gdb")
        pid = db.workflows.get_gdb_pid(wf_id)
        dep_manager.kill_gdb(pid)
        resp = make_response(jsonify(status='Cancelled'), 202)
        return resp

    def patch(self, wf_id):
        """Pause or resume workflow."""
        db = connect_db(wfm_db, db_path)
        self.reqparse.add_argument('option', type=str, location='json')
        option = self.reqparse.parse_args()['option']

        wfi = wf_utils.get_workflow_interface(wf_id)
        wf_state = wfi.get_workflow_state()
        if option == 'pause' and wf_state == 'RUNNING':
            wfi.pause_workflow()
            wf_utils.update_wf_status(wf_id, 'Paused')
            db.workflows.update_workflow_state(wf_id, 'Paused')
            log.info("Workflow Paused")
            resp = make_response(jsonify(status='Workflow Paused'), 200)
        elif option == 'resume' and wf_state == 'PAUSED':
            wfi.resume_workflow()
            tasks = wfi.get_ready_tasks()
            wf_utils.schedule_submit_tasks(wf_id, tasks)
            wf_utils.update_wf_status(wf_id, 'Running')
            db.workflows.update_workflow_state(wf_id, 'Running')
            log.info("Workflow Resumed")
            resp = make_response(jsonify(status='Workflow Resumed'), 200)
        else:
            resp_msg = f'Cannot {option} workflow. It is currently {wf_state.lower()}.'
            log.info(resp_msg)
            resp = make_response(jsonify(status=resp_msg), 200)
        return resp
